.Const

ECDrvName        	Equ      <ECDriver>
NT_DEVICE_NAME   	CatStr   <"\Device\>,ECDrvName,<">
DOS_DEVICE_NAME  	CatStr   <"\DosDevices\>,ECDrvName,<">

;--
FILE_DEVICE_ECDRV     Equ    8050H       ; Minimo valor 8000H
IOCTL_ECDRV_BASE      Equ    FILE_DEVICE_ECDRV
IOCTL_ECDRV_INDEX     Equ    850H        ; Minimo valor 800H
IOCTL_EDCRV_READCMOS  Equ    CTL_CODE(IOCTL_ECDRV_BASE, IOCTL_ECDRV_INDEX, \
                                      METHOD_BUFFERED, FILE_ANY_ACCESS)
IOCTL_EDCRV_BEEP      Equ    CTL_CODE(IOCTL_ECDRV_BASE, IOCTL_ECDRV_INDEX + 1, \
                                      METHOD_BUFFERED, FILE_ANY_ACCESS)
IOPM_SIZE             Equ    2000H
;--

.Data?

DEVICE_EXTENSION Struct
;This structure is driver-defined.
;It must be filled depending on
;the driver to be programmed.
DEVICE_EXTENSION EndS

.Data

.Code
MakeBeep Proc Freq:DWord
                 Invoke HalMakeBeep, Freq
                   Mov Eax, 1800000H * 2
                   .While Eax
                       Dec Eax
                   .EndW
                 Invoke HalMakeBeep, 0
                 Ret
MakeBeep EndP

ReadCMOS Proc Uses Ebx Ecx BufPtr:DWord, BufLen:DWord, PID:DWord
              Local pIopm:PVOID, pProcess:PVOID

           Invoke MmAllocateNonCachedMemory, IOPM_SIZE           ; Mem Alloc
           .If Eax != NULL
                Mov pIopm, Eax
                Invoke PsLookupProcessByProcessId, PID, Addr pProcess
               .If Eax == STATUS_SUCCESS
                   Invoke Ke386QueryIoAccessMap, 1, pIopm        ; Get IOPL
                   .If Al != 0
                        Invoke RtlZeroMemory, pIopm, IOPM_SIZE   ; Reset IOPL
                        Invoke Ke386SetIoAccessMap, 1, pIopm     ; Set modified IOPM
                        .If Al != 0
                           Invoke Ke386IoSetAccessProcess, pProcess, 1
                        .EndIf
                   .EndIf
                   Invoke ObDereferenceObject, pProcess
               .EndIf
            Invoke MmFreeNonCachedMemory, pIopm, IOPM_SIZE       ; Mem Free
           .EndIf
     ;------------------------------------------------------------
             Xor Ecx, Ecx                           ; Ecx == 0
             Mov Ebx, BufPtr
             .While Ecx < 80H                       ; 127 bytes
                Mov Al, Cl
                Out 70H, Al
                ;Invoke WRITE_PORT_UCHAR, 70H, Ecx
                In Al, 71H
                ;Invoke READ_PORT_UCHAR, 71H
                Mov [Ebx + Ecx], Al
                Inc Ecx
             .EndW
      ;--
      Ret
ReadCMOS EndP

DriverEntry Proc pDriverObject:PDRIVER_OBJECT, pusRegistryPath:PUNICODE_STRING
	Local usDeviceName:UNICODE_STRING, usSymbolicLinkName:UNICODE_STRING
	Local pDeviceObject:PDEVICE_OBJECT

	Invoke RtlInitUnicodeString, Addr usDeviceName, TextAddrW(%NT_DEVICE_NAME)
	Invoke RtlInitUnicodeString, Addr usSymbolicLinkName, TextAddrW(%DOS_DEVICE_NAME)

	Invoke IoCreateDevice, pDriverObject, SizeOf DEVICE_EXTENSION, Addr usDeviceName, \
							FILE_DEVICE_UNKNOWN, 0, TRUE, Addr pDeviceObject
	.If Eax != STATUS_SUCCESS
		Mov Eax, STATUS_DEVICE_CONFIGURATION_ERROR
		Ret
	.EndIf

	Invoke IoCreateSymbolicLink, Addr usSymbolicLinkName, Addr usDeviceName
	.If Eax != STATUS_SUCCESS
		Invoke IoDeleteDevice, pDriverObject
		Mov Eax, STATUS_DEVICE_CONFIGURATION_ERROR
		Ret
	.EndIf

	Mov Eax, pDriverObject
	Mov (DRIVER_OBJECT Ptr [Eax]).DriverUnload, Offset DriverUnload

	Mov (DRIVER_OBJECT Ptr [Eax]).MajorFunction[IRP_MJ_CREATE * (SizeOf PVOID)], Offset DriverDispatch
	Mov (DRIVER_OBJECT Ptr [Eax]).MajorFunction[IRP_MJ_CLOSE * (SizeOf PVOID)], Offset DriverDispatch
	Mov (DRIVER_OBJECT Ptr [Eax]).MajorFunction[IRP_MJ_DEVICE_CONTROL * (SizeOf PVOID)], Offset DriverDispatch
	Mov (DRIVER_OBJECT Ptr [Eax]).MajorFunction[IRP_MJ_READ * (SizeOf PVOID)], Offset DriverDispatch
	Mov (DRIVER_OBJECT Ptr [Eax]).MajorFunction[IRP_MJ_WRITE * (SizeOf PVOID)], Offset DriverDispatch

	Mov Eax, STATUS_SUCCESS
	Ret
DriverEntry EndP

DriverUnload Proc pDriverObject:PDRIVER_OBJECT
	Local usSymbolicLinkName:UNICODE_STRING

	Invoke RtlInitUnicodeString, Addr usSymbolicLinkName, TextAddrW(%DOS_DEVICE_NAME)
	Invoke IoDeleteSymbolicLink, Addr usSymbolicLinkName

	Mov Eax, pDriverObject
	Invoke IoDeleteDevice, (DRIVER_OBJECT Ptr [Eax]).DeviceObject
	Ret
DriverUnload EndP

DriverDispatch Proc Uses Ecx Edx pDeviceObject:PDEVICE_OBJECT, pIrp:PIRP
	Local Status:NTSTATUS, Info:DWord, IO_S_L:DWord

    Local IOCBuff_Ptr:DWord           ; I/O Control Buffer
    Local ICBuff_Len:DWord            ; Input Control Buffer Len
    Local OCBuff_Len:DWord            ; Out Control Buffer Len
    Local FilePointer:LARGE_INTEGER   ; R/W File Pointer

	Mov Info, 0H
	Mov Eax, pIrp
	Mov Eax, (_IRP Ptr [Eax]).Tail.Overlay.CurrentStackLocation
	Mov IO_S_L, Eax

	Movzx Eax, (IO_STACK_LOCATION Ptr [Eax]).MajorFunction

	.If Eax == IRP_MJ_CREATE
		Mov Status, STATUS_SUCCESS
	.ElseIf Eax == IRP_MJ_CLOSE
		Mov Status, STATUS_SUCCESS
	.ElseIf Eax == IRP_MJ_DEVICE_CONTROL
           Mov Eax, pIrp
           Move IOCBuff_Ptr, (_IRP Ptr [Eax]).AssociatedIrp.SystemBuffer   ; Buff Ptr

           Mov Eax, IO_S_L
           Move ICBuff_Len, (IO_STACK_LOCATION Ptr [Eax]).Parameters.DeviceIoControl.InputBufferLength  ; Inp Buff Len
           Move OCBuff_Len, (IO_STACK_LOCATION Ptr [Eax]).Parameters.DeviceIoControl.OutputBufferLength ; Out Buff Len
           ;Mov Edx, (IO_STACK_LOCATION Ptr [Eax]).Parameters.DeviceIoControl.Type3InputBuffer   ; InputBuffer

           Mov Eax, (IO_STACK_LOCATION Ptr [Eax]).Parameters.DeviceIoControl.IoControlCode              ; Control

           .If Eax == IOCTL_EDCRV_READCMOS     ; Control
              Mov Eax, IOCBuff_Ptr
              Mov Edx, [Eax]                   ; PID
              Mov [Eax + 150], Edx             ; PID en Out + 150
              Invoke ReadCMOS, IOCBuff_Ptr, OCBuff_Len, Edx
              Move Info, OCBuff_Len
           .ElseIf Eax == IOCTL_EDCRV_BEEP     ; Control
              Invoke MakeBeep, 1000
              Mov Info, 0
           .EndIf
		Mov Status, STATUS_SUCCESS
	.ElseIf Eax == IRP_MJ_READ
           Mov Eax, pIrp
           Move IOCBuff_Ptr, (_IRP Ptr [Eax]).UserBuffer                   ; Buff Ptr

           Mov Eax, IO_S_L
           Move OCBuff_Len, (IO_STACK_LOCATION Ptr [Eax]).Parameters.Read.dwLength  ; READ Out Buff Len
           Move FilePointer.LowPart, (IO_STACK_LOCATION Ptr [Eax]).Parameters.Read.ByteOffset.LowPart  ; READ File Pointer
           Move FilePointer.HighPart, (IO_STACK_LOCATION Ptr [Eax]).Parameters.Read.ByteOffset.LowPart ; READ File Pointer
           ;--
             Invoke RtlZeroMemory, IOCBuff_Ptr, OCBuff_Len
             Invoke RtlMoveMemory, IOCBuff_Ptr, TextAddr("Driver Respondiendo READ"), 24
             Move Info, OCBuff_Len
           ;--
		Mov Status, STATUS_SUCCESS
	.ElseIf Eax == IRP_MJ_WRITE
		Mov Status, STATUS_SUCCESS
	.Else
		Mov Status, STATUS_NOT_IMPLEMENTED
				.EndIf

	Mov Eax, pIrp
	Move (_IRP Ptr [Eax]).IoStatus.Status, Status
	Move (_IRP Ptr [Eax]).IoStatus.Information, Info

    ;IoCompleteRequest is suposed to be defined as fastcall,
    ;which means the first two parameters have to be passed
    ;in Ecx and Edx registers.
	Mov Edx, IO_NO_INCREMENT
	Mov Ecx, pIrp
	Call IoCompleteRequest
	Mov Eax, Status
	Ret
DriverDispatch EndP

End DriverEntry
