;EasyCodeName=ECIDTDriver,1
.Const

ECDrvName        	Equ      <ECIDTDriver>
NT_DEVICE_NAME   	CatStr   <"\Device\>,ECDrvName,<">
DOS_DEVICE_NAME  	CatStr   <"\DosDevices\>,ECDrvName,<">

.Data?

DEVICE_EXTENSION Struct
;This structure is driver-defined.
;It must be filled depending on
;the driver to be programmed.
DEVICE_EXTENSION EndS

IDT Struc
   Limit       DW    ?
   Base        DD    ?
IDT EndS

.Data
IDT_          IDT   <0>

.Code

DriverEntry Proc pDriverObject:PDRIVER_OBJECT, pusRegistryPath:PUNICODE_STRING
	Local usDeviceName:UNICODE_STRING, usSymbolicLinkName:UNICODE_STRING
	Local pDeviceObject:PDEVICE_OBJECT

	Invoke RtlInitUnicodeString, Addr usDeviceName, TextAddrW(%NT_DEVICE_NAME)
	Invoke RtlInitUnicodeString, Addr usSymbolicLinkName, TextAddrW(%DOS_DEVICE_NAME)

	Invoke IoCreateDevice, pDriverObject, SizeOf DEVICE_EXTENSION, Addr usDeviceName, \
							FILE_DEVICE_UNKNOWN, 0, TRUE, Addr pDeviceObject
	.If Eax != STATUS_SUCCESS
		Mov Eax, STATUS_DEVICE_CONFIGURATION_ERROR
		Ret
	.EndIf

	Invoke IoCreateSymbolicLink, Addr usSymbolicLinkName, Addr usDeviceName
	.If Eax != STATUS_SUCCESS
		Invoke IoDeleteDevice, pDriverObject
		Mov Eax, STATUS_DEVICE_CONFIGURATION_ERROR
		Ret
	.EndIf

	Mov Eax, pDriverObject
	Mov (DRIVER_OBJECT Ptr [Eax]).DriverUnload, Offset DriverUnload

	Mov (DRIVER_OBJECT Ptr [Eax]).MajorFunction[IRP_MJ_CREATE * (SizeOf PVOID)], Offset DriverDispatch
	Mov (DRIVER_OBJECT Ptr [Eax]).MajorFunction[IRP_MJ_CLOSE * (SizeOf PVOID)], Offset DriverDispatch
	Mov (DRIVER_OBJECT Ptr [Eax]).MajorFunction[IRP_MJ_DEVICE_CONTROL * (SizeOf PVOID)], Offset DriverDispatch
	Mov (DRIVER_OBJECT Ptr [Eax]).MajorFunction[IRP_MJ_READ * (SizeOf PVOID)], Offset DriverDispatch
	Mov (DRIVER_OBJECT Ptr [Eax]).MajorFunction[IRP_MJ_WRITE * (SizeOf PVOID)], Offset DriverDispatch

	Mov Eax, STATUS_SUCCESS
	Ret
DriverEntry EndP

DriverUnload Proc pDriverObject:PDRIVER_OBJECT
	Local usSymbolicLinkName:UNICODE_STRING

	Invoke RtlInitUnicodeString, Addr usSymbolicLinkName, TextAddrW(%DOS_DEVICE_NAME)
	Invoke IoDeleteSymbolicLink, Addr usSymbolicLinkName

	Mov Eax, pDriverObject
	Invoke IoDeleteDevice, (DRIVER_OBJECT Ptr [Eax]).DeviceObject
	Ret
DriverUnload EndP

DriverDispatch Proc Uses Ecx Edx pDeviceObject:PDEVICE_OBJECT, pIrp:PIRP
	Local Status:NTSTATUS, Info:DWord, IO_S_L:DWord

    Local RBufPtr:DWord           ; Buffer
    Local RBufLen:SDWord          ; Buffer Len

	Mov Info, 0H
	Mov Eax, pIrp
	Mov Eax, (_IRP Ptr [Eax]).Tail.Overlay.CurrentStackLocation
	Mov IO_S_L, Eax

	Movzx Eax, (IO_STACK_LOCATION Ptr [Eax]).MajorFunction

	.If Eax == IRP_MJ_CREATE
		Mov Status, STATUS_SUCCESS
	.ElseIf Eax == IRP_MJ_CLOSE
		Mov Status, STATUS_SUCCESS
	.ElseIf Eax == IRP_MJ_DEVICE_CONTROL
		Mov Status, STATUS_SUCCESS
	.ElseIf Eax == IRP_MJ_READ
		;---
            Mov Eax, pIrp
            Move RBufPtr, (_IRP Ptr [Eax]).UserBuffer                   ; Buff Ptr
            ;Move RBufPtr, (_IRP Ptr [Eax]).AssociatedIrp.SystemBuffer   ; Buff Ptr
            Mov Eax, IO_S_L
            Move RBufLen, (IO_STACK_LOCATION Ptr [Eax]).Parameters.Read.dwLength  ; READ Out Buff Len
            ;--
            Invoke  Read_IDT, RBufPtr, RBufLen
            Move Info, RBufLen
            Mov Status, Eax
            ;--
		;---
		;Mov Status, STATUS_SUCCESS
	.ElseIf Eax == IRP_MJ_WRITE
		Mov Status, STATUS_SUCCESS
	.Else
		Mov Status, STATUS_NOT_IMPLEMENTED
	.EndIf

	Mov Eax, pIrp
	Move (_IRP Ptr [Eax]).IoStatus.Status, Status
	Move (_IRP Ptr [Eax]).IoStatus.Information, Info

    ;IoCompleteRequest is suposed to be defined as fastcall,
    ;which means the first two parameters have to be passed
    ;in Ecx and Edx registers.
	Mov Edx, IO_NO_INCREMENT
	Mov Ecx, pIrp
	Call IoCompleteRequest
	Mov Eax, Status
	Ret

DriverDispatch EndP

Read_IDT Proc Uses Ebx BufPtr:DWord, BufLen:DWord
    Sidt IDT_
    Movzx Eax, IDT_.Limit
    ; first two DWORD will be Idt.Base and Idt.Limit
    .If Eax <= BufLen - SizeOf DWord * 2
        Mov Ebx, BufPtr
        Mov Eax, IDT_.Base
        Mov [Ebx], Eax
        Movzx Eax, IDT_.Limit
        Mov [Ebx + 4], Eax
        Add Ebx, SizeOf DWord * 2
        Invoke RtlMoveMemory, Ebx, IDT_.Base, Eax
        Movzx Eax, IDT_.Limit
        ; The IDT limit is always be one less than an integral multiple of eight (th
        Inc Eax
        Add Eax, SizeOf DWord * 2
        Mov Eax, STATUS_SUCCESS
     .Else
     	Mov Eax, STATUS_BUFFER_TOO_SMALL
     .EndIf
	Ret
Read_IDT EndP

End DriverEntry
